package org.appfuse.dao;

import java.util.List;

import org.appfuse.model.User;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.dao.DataAccessException;

public class UserDAOTest extends BaseDAOTestCase {

    private User user = null;
    private UserDAO dao = null;

    /* Option 1: Reload context and database each time.
     * Also Option 3: Static initialization block and 
     * reload database each time. */
    protected void setUp() throws Exception {
    	super.setUp();
        dao = (UserDAO) ctx.getBean("userDAO");
    }

    protected void tearDown() throws Exception {
    	super.tearDown();
        dao = null;
    }
    
    /* Option 2: Use JUnit's TestSetup class 
    public static Test suite() {
        return new TestSetup(new TestSuite(UserDAOTest.class)) {
                public void setUp() throws Exception {
                    start();
                    dao = (UserDAO) ctx.getBean("userDAO");
                }

                public void tearDown() throws Exception {
                    stop();
                    dao = null;
                }
            };
    }*/

    public void testGetUsers() {
        List users = dao.getUsers();
        assertTrue(users.size() == 1);

        User user = (User) users.get(0);
        assertEquals(user.getFullName(), "Rod Johnson");
    }
    
    public void testSaveUser() throws Exception {
        user = new User();
        user.setFirstName("Rod");
        user.setLastName("Johnson");

        dao.saveUser(user);
        assertTrue("primary key assigned", user.getId() != null);
        log.info(user);
        assertTrue(user.getFirstName() != null);
    }

    public void testAddAndRemoveUser() throws Exception {
        user = new User();
        user.setFirstName("Bill");
        user.setLastName("Joy");

        dao.saveUser(user);

        assertTrue(user.getId() != null);
        assertTrue(user.getFirstName().equals("Bill"));

        if (log.isDebugEnabled()) {
            log.debug("removing user...");
        }

        dao.removeUser(user.getId());

        try {
            user = dao.getUser(user.getId());
            fail("User found in database");
        } catch (DataAccessException dae) {
            if (log.isDebugEnabled()) {
                log.debug("Expected exception: " + dae.getMessage());
            }

            assertTrue(dae != null);
        }
    }
}
