package org.appfuse.web;

import java.util.ArrayList;
import java.util.Map;

import org.apache.cactus.ServletTestCase;
import org.apache.cactus.WebRequest;
import org.appfuse.model.User;
import org.appfuse.service.UserManager;
import org.easymock.MockControl;
import org.springframework.context.ApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;
import org.springframework.web.servlet.ModelAndView;

public class UserCactusEMTest extends ServletTestCase {
    private UserController list = new UserController();
    private UserFormController form = new UserFormController();
    private MockControl control = null;
    private UserManager mockManager = null;
    
    protected void setUp() throws Exception {
        control = MockControl.createControl(UserManager.class);
        mockManager = (UserManager) control.getMock();
        list.setUserManager(mockManager);
        form.setUserManager(mockManager);

        // needed to prevent NPE with getMessageSourceAccessor() in form
        ApplicationContext ctx = 
            WebApplicationContextUtils
                .getRequiredWebApplicationContext(
                        session.getServletContext());
        form.setApplicationContext(ctx);;
    }

    public void beginAddUser(WebRequest wRequest) {
        wRequest.addParameter("firstName", "Dion", "post");
        wRequest.addParameter("lastName", "Almaer", "post");
    }
    
    public void testAddUser() throws Exception {
        // set expected behavior on manager
        User user = new User();
        user.setFirstName("Dion");
        user.setLastName("Almaer");
        mockManager.saveUser(user);
        control.setReturnValue(user);

        // switch from record to playback
        control.replay();

        form.handleRequest(request, response);

        // verify saveUser() was called
        control.verify();
        assertTrue(request.getSession().getAttribute("message") != null);
    }
    
    public void testUserList() throws Exception {
        // set expected behavior on manager
        control.expectAndReturn(mockManager.getUsers(), new ArrayList());
        // switch from record to playback
        control.replay();

        ModelAndView mav = list.handleRequest(request, response);

        // verify getUsers() was called
        control.verify();

        Map m = mav.getModel();
        assertNotNull(m.get("users"));
        assertEquals(mav.getViewName(), "userList");
    }
}